@@ -30,6 +30,8 @@
 #include <gcli/curl.h>
 #include <gcli/github/config.h>
 #include <gcli/github/issues.h>
+#include <gcli/github/milestones.h>
+#include <gcli/json_gen.h>
 #include <gcli/json_util.h>
 #include <pdjson/pdjson.h>
 
@@ -79,6 +81,48 @@ github_fetch_issues(gcli_ctx *ctx, char *url, int const max,
 	return gcli_fetch_list(ctx, url, &fl);
 }
 
+static int
+get_milestone_id(gcli_ctx *ctx, char const *owner, char const *repo,
+                 char const *milestone_name, gcli_id *out)
+{
+	int rc = 0;
+	gcli_milestone_list list = {0};
+
+	rc = github_get_milestones(ctx, owner, repo, -1, &list);
+	if (rc < 0)
+		return rc;
+
+	rc = gcli_error(ctx, "%s: no such milestone", milestone_name);
+
+	for (size_t i = 0; i < list.milestones_size; ++i) {
+		if (strcmp(list.milestones[i].title, milestone_name) == 0) {
+			*out = list.milestones[i].id;
+			rc = 0;
+			break;
+		}
+	}
+
+	gcli_free_milestones(&list);
+
+	return rc;
+}
+
+static int
+parse_github_milestone(gcli_ctx *ctx, char const *owner, char const *repo,
+                       char const *milestone, gcli_id *out)
+{
+	char *endptr = NULL;
+	size_t const m_len = strlen(milestone);
+
+	/* first try parsing as a milestone ID, if it isn't one,
+	 * go looking for a similarly named milestone */
+	*out = strtoull(milestone, &endptr, 10);
+	if (endptr == milestone + m_len)
+		return 0;
+
+	return get_milestone_id(ctx, owner, repo, milestone, out);
+}
+
 int
 github_get_issues(gcli_ctx *ctx, char const *owner, char const *repo,
                   gcli_issue_fetch_details const *details, int const max,
@@ -89,9 +133,18 @@ github_get_issues(gcli_ctx *ctx, char const *owner, char const *repo,
 	char *e_repo = NULL;
 	char *e_author = NULL;
 	char *e_label = NULL;
+	char *e_milestone = NULL;
 
-	e_owner = gcli_urlencode(owner);
-	e_repo  = gcli_urlencode(repo);
+	if (details->milestone) {
+		gcli_id milestone_id;
+		int rc;
+
+		rc = parse_github_milestone(ctx, owner, repo, details->milestone, &milestone_id);
+		if (rc < 0)
+			return rc;
+
+		e_milestone = sn_asprintf("&milestone=%"PRIid, milestone_id);
+	}
 
 	if (details->author) {
 		char *tmp = gcli_urlencode(details->author);
@@ -105,14 +158,19 @@ github_get_issues(gcli_ctx *ctx, char const *owner, char const *repo,
 		free(tmp);
 	}
 
+	e_owner = gcli_urlencode(owner);
+	e_repo  = gcli_urlencode(repo);
+
 	url = sn_asprintf(
-		"%s/repos/%s/%s/issues?state=%s%s%s",
+		"%s/repos/%s/%s/issues?state=%s%s%s%s",
 		gcli_get_apibase(ctx),
 		e_owner, e_repo,
 		details->all ? "all" : "open",
 		e_author ? e_author : "",
-		e_label ? e_label : "");
+		e_label ? e_label : "",
+		e_milestone ? e_milestone : "");
 
+	free(e_milestone);
 	free(e_author);
 	free(e_label);
 	free(e_owner);
@@ -135,11 +193,9 @@ github_get_issue_summary(gcli_ctx *ctx, char const *owner, char const *repo,
 	e_owner = gcli_urlencode(owner);
 	e_repo  = gcli_urlencode(repo);
 
-	url = sn_asprintf(
-		"%s/repos/%s/%s/issues/%lu",
-		gcli_get_apibase(ctx),
-		e_owner, e_repo,
-		issue_number);
+	url = sn_asprintf("%s/repos/%s/%s/issues/%"PRIid,
+	                  gcli_get_apibase(ctx), e_owner, e_repo,
+	                  issue_number);
 
 	rc = gcli_fetch(ctx, url, NULL, &buffer);
 
@@ -172,7 +228,7 @@ github_issue_close(gcli_ctx *ctx, char const *owner, char const *repo,
 	e_repo = gcli_urlencode(repo);
 
 	url = sn_asprintf(
-		"%s/repos/%s/%s/issues/%lu",
+		"%s/repos/%s/%s/issues/%"PRIid,
 		gcli_get_apibase(ctx),
 		e_owner, e_repo,
 		issue_number);
@@ -202,7 +258,7 @@ github_issue_reopen(gcli_ctx *ctx, char const *owner, char const *repo,
 	e_repo  = gcli_urlencode(repo);
 
 	url = sn_asprintf(
-		"%s/repos/%s/%s/issues/%lu",
+		"%s/repos/%s/%s/issues/%"PRIid,
 		gcli_get_apibase(ctx),
 		e_owner, e_repo,
 		issue_number);
@@ -266,7 +322,7 @@ github_issue_assign(gcli_ctx *ctx, char const *owner, char const *repo,
 	e_repo = gcli_urlencode(repo);
 
 	url = sn_asprintf(
-		"%s/repos/%s/%s/issues/%lu/assignees",
+		"%s/repos/%s/%s/issues/%"PRIid"/assignees",
 		gcli_get_apibase(ctx), e_owner, e_repo, issue_number);
 
 	rc = gcli_fetch_with_method(ctx, "POST", url, post_fields, NULL, NULL);
@@ -292,7 +348,7 @@ github_issue_add_labels(gcli_ctx *ctx, char const *owner, char const *repo,
 
 	assert(labels_size > 0);
 
-	url = sn_asprintf("%s/repos/%s/%s/issues/%lu/labels",
+	url = sn_asprintf("%s/repos/%s/%s/issues/%"PRIid"/labels",
 	                  gcli_get_apibase(ctx), owner, repo, issue);
 
 	list = sn_join_with(labels, labels_size, "\",\"");
@@ -323,7 +379,7 @@ github_issue_remove_labels(gcli_ctx *ctx, char const *owner, char const *repo,
 
 	e_label = gcli_urlencode(labels[0]);
 
-	url = sn_asprintf("%s/repos/%s/%s/issues/%lu/labels/%s",
+	url = sn_asprintf("%s/repos/%s/%s/issues/%"PRIid"/labels/%s",
 	                  gcli_get_apibase(ctx), owner, repo, issue, e_label);
 
 	rc = gcli_fetch_with_method(ctx, "DELETE", url, NULL, NULL, NULL);
@@ -345,11 +401,11 @@ github_issue_set_milestone(gcli_ctx *ctx, char const *const owner,
 	e_owner = gcli_urlencode(owner);
 	e_repo = gcli_urlencode(repo);
 
-	url = sn_asprintf("%s/repos/%s/%s/issues/%lu",
+	url = sn_asprintf("%s/repos/%s/%s/issues/%"PRIid,
 	                  gcli_get_apibase(ctx),
 	                  e_owner, e_repo, issue);
 
-	body = sn_asprintf("{ \"milestone\": %lu }", milestone);
+	body = sn_asprintf("{ \"milestone\": %"PRIid" }", milestone);
 
 	rc = gcli_fetch_with_method(ctx, "PATCH", url, body, NULL, NULL);
 
@@ -371,7 +427,7 @@ github_issue_clear_milestone(gcli_ctx *ctx, char const *const owner,
 	e_owner = gcli_urlencode(owner);
 	e_repo = gcli_urlencode(repo);
 
-	url = sn_asprintf("%s/repos/%s/%s/issues/%lu",
+	url = sn_asprintf("%s/repos/%s/%s/issues/%"PRIid,
 	                  gcli_get_apibase(ctx),
 	                  e_owner, e_repo, issue);
 
@@ -386,3 +442,42 @@ github_issue_clear_milestone(gcli_ctx *ctx, char const *const owner,
 
 	return rc;
 }
+
+int
+github_issue_set_title(gcli_ctx *ctx, char const *const owner,
+                       char const *const repo, gcli_id const issue,
+                       char const *const new_title)
+{
+	char *url, *e_owner, *e_repo, *payload;
+	gcli_jsongen gen = {0};
+	int rc;
+
+	/* Generate url */
+	e_owner = gcli_urlencode(owner);
+	e_repo = gcli_urlencode(repo);
+
+	url = sn_asprintf("%s/repos/%s/%s/issues/%"PRIid, gcli_get_apibase(ctx),
+	                  e_owner, e_repo, issue);
+
+	free(e_owner);
+	free(e_repo);
+
+	/* Generate payload */
+	gcli_jsongen_init(&gen);
+	gcli_jsongen_begin_object(&gen);
+	{
+		gcli_jsongen_objmember(&gen, "title");
+		gcli_jsongen_string(&gen, new_title);
+	}
+	gcli_jsongen_end_object(&gen);
+
+	payload = gcli_jsongen_to_string(&gen);
+	gcli_jsongen_free(&gen);
+
+	rc = gcli_fetch_with_method(ctx, "PATCH", url, payload, NULL, NULL);
+
+	free(payload);
+	free(url);
+
+	return rc;
+}
